-- AIRWINDOWMATION FOR REAPER AND CONTROL SURFACES
-- This is not actually automation, it's grouping.
-- It lets you tie any number of tracks to colorcoded faders
-- and swap them around all willy-nilly in mix.
-- as such, it's more for performing a mix than recording.
-- the default color is the same as using the color black.
-- if you want stuff not to chase the faders, you can
-- set the track colors to something not included below.

-- If you have your own colors, you can enter them here
-- this is using the Apple 11 color set on the Mac.
-- the three numbers specify the color of the track.
-- if you don't have these colors, change the colors below
-- to whatever you do have.

local colors = {
    {255,255,255}, -- 1 - White
    {251,2,255}, --   2 - Magenta
    {251,2,7}, --     3 - Red
    {253,128,8}, --   4 - Orange
    {255,255,10}, --  5 - Yellow
    {33,255,6}, --    6 - Green
    {33,255,255}, --  7 - Cyan
    {0,0,255}, --     8 - Blue
    {128,0,128}, --   9 - Purple
    {153,102,51}, --  10- Brown
    {0,0,0}, --       11- Black
    {1,2,3} --        12- Master Fader (doesn't use the supplied color)
} -- this is in the order I used for my control surface's faders
local volumes = {1,1,1,1,1,1,1,1,1,1,1,1,1,1}

-- you have to set up ReaLearn on the Monitoring FX chain
-- which is found at View:Monitoring FX, and you have to
-- rename that instance of ReaLearn to AirwindowsColorCoordinator.
-- it's set up to run 12 faders, 11 for colors and 12th for the master.
-- whatever MIDI CCs you use, assign them to the targets,

-- Type: FX Parameter: Set value
-- Track: <Master>
-- FX: Particular, AirwindowsColorCoordinator (this is itself!)
-- Parameter: Particular, Main p1: Param 1

-- Do that for CCs 1 through 12, there will be 12 entries in the mappings.

local Buss_Master_Fader = 12

-- specify which fader gets used for the master fader on ConsoleXBuss
-- this should have color entries that don't show up as track colors ever
-- if you want no master fader, set this to some number that will never
-- show up like 999 or something, and assign the colors all to real track colors.

-- if you have a control surface with a different number of faders than 12,
-- make colors (above) have the right number of entries, pick out enough
-- track colors to fit all the entries, and assign the control surface CCs
-- in ReaLearn (named AirwindowsColorCoordinator) to the params you'll have.

-------------------------------------------------------------

Current_Track = nil
Position_in_Monitoring_FX = nil
master_track = nil
param_value = 0


function onLaunch()
    master_track = reaper.GetMasterTrack(0)
    local monitoring_fx_count = reaper.TrackFX_GetRecCount(master_track) -- This is typically the monitoring FX count
    for fx = 0, monitoring_fx_count - 1 do
        local actual_fx_index = fx + 0x1000000 -- Add offset for Monitoring FX
        local retval, fx_name = reaper.TrackFX_GetFXName(master_track, actual_fx_index, "")
        if fx_name == "AirwindowsColorCoordinator" then
            Position_in_Monitoring_FX = actual_fx_index
        end
    end
    return nil
end -- here we set up access to ReaLearn


function setPluginParameter(Current_Track, param_value)
    local num_fx = reaper.TrackFX_GetCount(Current_Track)
    for fx_index = 0, num_fx - 1 do
        local retval, fx_name = reaper.TrackFX_GetFXName(Current_Track, fx_index, "")
        if fx_name == "VST3: ConsoleXChannel (Airwindows)" then
            reaper.TrackFX_SetParam(Current_Track, fx_index, 36, param_value)
            return
        end
    end
end -- here is where we update tracks to take the control surface value


function setMasterParameter(param_value)
    local num_fx = reaper.TrackFX_GetCount(master_track)
    for fx_index = 0, num_fx - 1 do
        local retval, fx_name = reaper.TrackFX_GetFXName(master_track, fx_index, "")
        if fx_name == "VST3: ConsoleXBuss (Airwindows)" then
            reaper.TrackFX_SetParam(master_track, fx_index, 36, param_value)
            return
        end
    end
end -- here is where we update the master fader to take the control surface value


local function matchPresetColor(r, g, b)
    for i, color in ipairs(colors) do
        if r == color[1] and g == color[2] and b == color[3] then
            return i
        end
    end
    return nil
end -- here we compare a track color with our list of control surface channels


-- Main loop
function loop()
    local num_tracks = reaper.CountTracks(0)
    for i, color in ipairs(colors) do
        volumes[i] = 1
    end
    for i = 0, num_tracks - 1 do
        local track = reaper.GetTrack(0, i)
        local r, g, b = reaper.ColorFromNative(reaper.GetTrackColor(track))
        local match = matchPresetColor(r, g, b)
        if match then
            volumes[match] = volumes[match] + 1
        end
    end
    for i = 0, num_tracks - 1 do
        local track = reaper.GetTrack(0, i)
        local r, g, b = reaper.ColorFromNative(reaper.GetTrackColor(track))
        local match = matchPresetColor(r, g, b)
        if match then
            param_value = reaper.TrackFX_GetParamNormalized(master_track, Position_in_Monitoring_FX, match-1) ^ volumes[match]
            if param_value then
                Current_Track = track
                setPluginParameter(Current_Track, param_value)
            end
        end
    end --we've now iterated through all tracks' colors setting 'fader' to the matching controller's fader color
    param_value = reaper.TrackFX_GetParamNormalized(master_track, Position_in_Monitoring_FX, Buss_Master_Fader-1) --master fader
    if param_value then
         setMasterParameter(param_value)
    end --we've now set the master fader to what the 12th controller fader is. Also constantly updated. 
    reaper.defer(loop) --once we turn Airwindowmation on, it just runs until Reaper is shut off
end

onLaunch()
reaper.defer(loop)
